/*
  HI65 - a high-level Commodore 65 emulator
  Copyright (C) 2013-2014  Simone Gremmo
  Contact: devilmaster@email.com

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  http://www.gnu.org/licenses/gpl.txt
*/

// display.cpp

#include <allegro.h>
#include <cstdio>
#include "f8x8.h"
#include "gdef.h"
#include "pro.h"
#include "var.h"

void Cout(uchar c)
{
 uchar msk,xtimes,ytimes;
 uint sy,x,y;
 uint add;
 
 if (c>=0xA0)
  c-=64;
 else
  c-=32;
  
 add=c;
 add<<=3; // add*=8;
 for (sy=0; sy<8; sy++) // vertical scan
 {
  y=CharY+sy*yzoom;
  if (!shifted)
   c=f8x8[add++];
  else
   c=f8x8shifted[add++];
  for (ytimes=0; ytimes<yzoom; ytimes++)
  {
   x=CharX;
   msk=0x80;
   while(msk)
   {
    for (xtimes=0; xtimes<xzoom; xtimes++)
    {
     (c&msk) ? _putpixel32(screen,x+screen_border_x,y+screen_border_y,forecolor) : _putpixel32(screen,x+screen_border_x,y+screen_border_y,backcolor);
     x++;
    }
    msk>>=1;
   }
   y++;
  }
 }
 // next access
 CharX+=(xzoom<<3); // CharX+=8*xzoom;
}

void prints(uchar *str)
{
 uchar c;
 
 // acquire_screen();
 while ((c=*str++)!=0)
  Cout(c);
 // release_screen();
}

void printn(uint number) // it prints integer numbers
{
 uchar asciinumber[50];
 uchar i,shiftedbak,howmanydigits;

 shiftedbak=shifted;
 if (shifted)
  shifted=0;
 howmanydigits=sprintf((char *)asciinumber,"%d",number);
 for (i=0; i<howmanydigits; i++)
  Cout(asciinumber[i]);
 shifted=shiftedbak;
}

void printn(double number) // it prints double precision floating point numbers
{
 uchar asciinumber[50];
 uchar i,shiftedbak,howmanydigits;

 shiftedbak=shifted;
 if (shifted)
  shifted=0;
 howmanydigits=sprintf((char *)asciinumber,"%f",number);
 for (i=0; i<howmanydigits; i++)
  Cout(asciinumber[i]);
 shifted=shiftedbak;
}

void printerror(double error)
{
 uint i, aux, scanned_line_number, scanned_instruction_end;
 
 if (!trap) // if a trap line has NOT been set, we print the error...
 {
  numvars[ER][0]=error;
  numvars[EL][0]=line_number;
  aux=(uint)(21*(numvars[ER][0]-1));
 
  Enter();
  if (screentype==65)
   forecolor=makecol(240,0,0);
  // acquire_screen();
  Cout ('?');
  for (i=aux; !(c65basicerrors[i]==' ' && c65basicerrors[i+1]==' '); i++)
   Cout(c65basicerrors[i]);
  // release_screen();
  prints ((uchar *)" error in ");
  printn ((uint)(numvars[EL][0]));
  if (screentype==65)
   forecolor=makecol(240,240,240);
 }
 else // if we do have a trap line, we redirect the execution to that line instead
 {
  // this is a variation of the GOTO code
  if (trap<=line_number)
   pc=0;
 
  do
  {
   // We skip to the end of the instruction
   // If we are at the beginning of the program, we consider the second
   // char (where pc==1) as the end of a hypothetical "instruction #0"
   // otherwise we advance to the actual end of the instruction
   if (!pc)
    pc++;
   else
   {
    while (program[pc])
     pc++;
   }
   scanned_instruction_end=pc;
    
   // We move 3 bytes forward, to "land" on the first byte of the line number
   pc+=3;
  
   // We read the line number (tokenized LSB-first = little endian)
   scanned_line_number=program[pc];
   pc++;
   aux=program[pc];
   aux<<=8;
   scanned_line_number=aux+scanned_line_number;
  
   // The MSB of the line number may be 0 (which would be interpreted
   // as an instruction end) so we move 1 step forward
   pc++;
  }
  while (scanned_line_number<trap);
 
  // Now we found the correct line. We must return to the beginning of
  // the instruction
  pc=scanned_instruction_end+1;
 }
}

void forebackswap(void)
{
 uint copycolor;
 
 copycolor=forecolor;
 forecolor=backcolor;
 backcolor=copycolor;
}

void BackSpace(void)
{
 if (CharX)
  CharX-=(xzoom<<3);
  
 // acquire_screen();
 rectfill(screen,CharX+screen_border_x,CharY+screen_border_y,CharX+(xzoom<<3)+screen_border_x,CharY+(yzoom<<3)+screen_border_y,backcolor);
 // release_screen();
}

void Enter(void)
{
 CharX=0;
 CharY+=yzoom<<3;
 
 if (screen_height==720 || screen_height==1080)
 {
  if (CharY>168*yzoom)
   CharY=0;
 }
 else
 {
  if (CharY>192*yzoom)
   CharY=0;
 }
}

void Tab(void)
{
 uint cx;
 
 cx=CharX/(xzoom<<3); // the # of pixels is converted to the # of characters
 while (cx%10)
  cx++;
  
 CharX=cx*(xzoom<<3); // the # of characters is converted to the # of pixels
}

void c65draw(uint x, uint y)
{
 // The main pixel is plotted
 _putpixel32(screen,x+screen_border_x,y+screen_border_y,forecolor);
  
 // Additional pixels must be plotted to simulate different resolutions
 if (xzoom>=2)
  _putpixel32(screen,x+screen_border_x+1,y+screen_border_y,forecolor);
 if (yzoom>=2)
  _putpixel32(screen,x+screen_border_x,y+screen_border_y+1,forecolor);
 if (xzoom>=2 && yzoom>=2)
  _putpixel32(screen,x+screen_border_x+1,y+screen_border_y+1,forecolor);
 if (xzoom==4)
  {
   _putpixel32(screen,x+screen_border_x+2,y+screen_border_y,forecolor);
   _putpixel32(screen,x+screen_border_x+3,y+screen_border_y,forecolor);
   _putpixel32(screen,x+screen_border_x+2,y+screen_border_y+1,forecolor);
   _putpixel32(screen,x+screen_border_x+3,y+screen_border_y+1,forecolor);
  }
  if (yzoom==4)
  {
   _putpixel32(screen,x+screen_border_x,y+screen_border_y+2,forecolor);
   _putpixel32(screen,x+screen_border_x,y+screen_border_y+3,forecolor);
   _putpixel32(screen,x+screen_border_x+1,y+screen_border_y+2,forecolor);
   _putpixel32(screen,x+screen_border_x+1,y+screen_border_y+3,forecolor);
  }
  if (xzoom==4 && yzoom==4)
  {
   _putpixel32(screen,x+screen_border_x+2,y+screen_border_y+2,forecolor);
   _putpixel32(screen,x+screen_border_x+3,y+screen_border_y+2,forecolor);
   _putpixel32(screen,x+screen_border_x+2,y+screen_border_y+3,forecolor);
   _putpixel32(screen,x+screen_border_x+3,y+screen_border_y+3,forecolor);
  }
}

void paletterestore(void)
{
 uint i;
 
 for (i=0; i<256; i+=16)
 {
  palette[0][0+i][0]= c65color[0];  palette[0][0+i][1]= c65color[0];  palette[0][0+i][2]= c65color[0];  // black
  palette[0][1+i][0]= c65color[15]; palette[0][1+i][1]= c65color[15]; palette[0][1+i][2]= c65color[15]; // white
  palette[0][2+i][0]= c65color[15]; palette[0][2+i][1]= c65color[0];  palette[0][2+i][2]= c65color[0];  // red
  palette[0][3+i][0]= c65color[0];  palette[0][3+i][1]= c65color[15]; palette[0][3+i][2]= c65color[15]; // cyan
  palette[0][4+i][0]= c65color[15]; palette[0][4+i][1]= c65color[0];  palette[0][4+i][2]= c65color[15]; // magenta
  palette[0][5+i][0]= c65color[0];  palette[0][5+i][1]= c65color[15]; palette[0][5+i][2]= c65color[0];  // green
  palette[0][6+i][0]= c65color[0];  palette[0][6+i][1]= c65color[0];  palette[0][6+i][2]= c65color[15]; // blue
  palette[0][7+i][0]= c65color[15]; palette[0][7+i][1]= c65color[15]; palette[0][7+i][2]= c65color[0];  // yellow
  palette[0][8+i][0]= c65color[15]; palette[0][8+i][1]= c65color[6];  palette[0][8+i][2]= c65color[0];  // orange
  palette[0][9+i][0]= c65color[10]; palette[0][9+i][1]= c65color[4];  palette[0][9+i][2]= c65color[0];  // brown
  palette[0][10+i][0]=c65color[15]; palette[0][10+i][1]=c65color[7];  palette[0][10+i][2]=c65color[7];  // pink
  palette[0][11+i][0]=c65color[5];  palette[0][11+i][1]=c65color[5];  palette[0][11+i][2]=c65color[5];  // dark gray
  palette[0][12+i][0]=c65color[8];  palette[0][12+i][1]=c65color[8];  palette[0][12+i][2]=c65color[8];  // medium gray
  palette[0][13+i][0]=c65color[9];  palette[0][13+i][1]=c65color[15]; palette[0][13+i][2]=c65color[9];  // light green
  palette[0][14+i][0]=c65color[9];  palette[0][14+i][1]=c65color[9];  palette[0][14+i][2]=c65color[15]; // light blue
  palette[0][15+i][0]=c65color[11]; palette[0][15+i][1]=c65color[11]; palette[0][15+i][2]=c65color[11]; // light gray
 }
}

void simulateloadinstructions(uchar *str, uchar runprogram)
{
 uchar c;
 uchar *other_str;
 
 other_str=str;
 if (runprogram)
 {
  prints((uchar *)"dload ");
  // acquire_screen();
  Cout('"');
  
  if (str[1]==':')
   shifted=1;
  
  while ((c=*str++)!=0)
   Cout(c);
  
  // if the screen type imitates a PET screen, we must NOT return to
  // the unshifted font 
  if (shifted && screentype)
   shifted=0;
   
  Cout('"');
  // release_screen();
  Enter();
 }
 Enter();
 prints((uchar *)"searching for ");
 // acquire_screen();
 
 if (other_str[1]==':')
   shifted=1;
 
 while ((c=*other_str++)!=0)
  Cout(c);
  
 if (shifted && screentype)
   shifted=0;
  
 // release_screen();
 if (!loaderror)
  Ready();
 else
 {
  Enter();
  forecolor=makecol(240,0,0);
  prints((uchar *)"?dos: file not found");
  forecolor=makecol(240,240,240);
 }
}

void Ready(void)
{
 Enter();
 prints((uchar *)"ready.");
 Enter();
}

int startscreen(uchar from_command)
{
 uchar i,factor;
 int gfxerror=0;
 
 if (screen_width==1280 || screen_width==1366 || screen_width==1600)
 {
  xzoom=2;
  yzoom=4;
  factor=4;
 }
 else if(screen_width==1920)
 {
  xzoom=3;
  yzoom=6;
  factor=5;
 }
 else
 {
  xzoom=1;
  yzoom=2;
  factor=3;
 }
 
 // The initialization is only done if the function has NOT been called
 // from a BASIC command
 if (!from_command)
 {
  // set the color palette
  // set_palette(desktop_palette);
  set_color_depth(32);

  // set a graphic mode
  if (fullscreen)
   gfxerror=set_gfx_mode(GFX_AUTODETECT,screen_width,screen_height,0,0);
  else
   gfxerror=set_gfx_mode(GFX_AUTODETECT_WINDOWED,screen_width,screen_height,0,0);
  
  if (gfxerror)
  {
   if (set_gfx_mode(GFX_SAFE,screen_width,screen_height,0,0))
   {
    set_gfx_mode(GFX_TEXT,0,0,0,0);
    allegro_message("Unable to set any graphic mode\n%s\n", allegro_error);
    return 1;
   }
  }
 }
 
 backcolor=makecol(0,0,240);
 // clear the screen to blue
 clear_to_color(screen,backcolor);

 /* you don't need to do this, but on some platforms (eg. Windows) things
 * will be drawn more quickly if you always acquire the screen before
 * trying to draw onto it.
 */
 acquire_screen();

 // create C65 screen
 backcolor=makecol(247,0,0);     CharX=0; CharY=(0<<factor)<<1; prints((uchar *)"                     "); forecolor=backcolor; backcolor=makecol(0,0,240); Cout(0xA9);
 backcolor=makecol(247,115,115); CharX=0; CharY=(1<<factor)<<1; prints((uchar *)"                  "); forecolor=backcolor; backcolor=makecol(0,0,240); Cout(0xA9);
 backcolor=makecol(247,247,0);   CharX=0; CharY=(2<<factor)<<1; prints((uchar *)"               "); forecolor=backcolor; backcolor=makecol(0,0,240); Cout(0xA9);
 backcolor=makecol(0,247,0);     CharX=0; CharY=(3<<factor)<<1; prints((uchar *)"            "); forecolor=backcolor; backcolor=makecol(0,0,240); Cout(0xA9);
 backcolor=makecol(148,148,247); CharX=0; CharY=(4<<factor)<<1; prints((uchar *)"          "); forecolor=backcolor; backcolor=makecol(0,0,240); Cout(0xA9);
 backcolor=makecol(247,0,247);   CharX=0; CharY=(5<<factor)<<1; prints((uchar *)"        "); forecolor=backcolor; backcolor=makecol(0,0,240); Cout(0xA9);
 
 forecolor=makecol(240,240,240);
 backcolor=makecol(0,0,240);
 CharX=30<<factor; CharY=(1<<factor)<<1;  prints((uchar *)"hi65: the high level commodore 65 emulator");
 CharX=34<<factor; CharY=(3<<factor)<<1;  prints((uchar *)"copyright  2013-2015  ");
 for (i=84; i<88; i++)
  Cout(c65basicerrors[i]);
 Cout(c65basicerrors[23]); CharX+=(1<<factor);
 Cout(c65basicerrors[4]); Cout(c65basicerrors[5]);
 Cout(c65basicerrors[462]); Cout(c65basicerrors[463]);
 Cout(c65basicerrors[187]); Cout(c65basicerrors[188]);
 CharX=27<<factor; CharY=(5<<factor)<<1;  prints((uchar *)" basic 10.0     version 6    all rights reserved");
 CharX=0;  CharY=(7<<factor)<<1;  prints((uchar *)"default keyboard");
 CharX=0;  CharY=(8<<factor)<<1; prints((uchar *)"expansion ram");
 CharX=0;  CharY=(9<<factor)<<1; prints((uchar *)"muparser ");
 if (enable_muparser)
  prints((uchar *)"enabled");
 else
  prints((uchar *)"disabled");
 CharX=0;  CharY=(10<<factor)<<1; 
 // you must always release bitmaps before calling any input functions
 // release_screen();
 
 return 0;
}

void makeborder(uint bordercolor) // so we can make a border from outside a BASIC 10 program too
{
 // acquire_screen();
 switch (screen_height)
 {
  case 600:
  case 900:
       rectfill(screen,0,screen_border_y,screen_border_x-1,screen_height-screen_border_y,bordercolor);
       rectfill(screen,screen_width-screen_border_x,screen_border_y,screen_width,screen_height-screen_border_y,bordercolor);
  case 480:
       rectfill(screen,0,0,screen_width,screen_border_y,bordercolor);
       rectfill(screen,0,screen_height-screen_border_y,screen_width,screen_height,bordercolor);
       break;
  case 768:
       rectfill(screen,0,screen_border_y,screen_border_x-1,screen_height-screen_border_y,bordercolor);
       rectfill(screen,screen_width-screen_border_x,screen_border_y,screen_width,screen_height-screen_border_y,bordercolor);
       break;
 }
 // release_screen();
}

void alternatescreen()
{
 uchar factor,i;
 uint year=2015;
 
 switch (screentype)
 {
  case 0:
       forecolor=makecol(0,255,0);
       backcolor=makecol(0,0,0);
       break;
  case 16:
       forecolor=makecol(187,163,255);
       backcolor=makecol(228,228,228);
       break;
  case 20:
       forecolor=makecol(92,203,216);
       backcolor=makecol(255,255,255);
       break;
  case 64:
       forecolor=makecol(144,144,240);
       backcolor=makecol(0,0,240);
       break;
  case 128:
       forecolor=makecol(179,236,145);
       backcolor=makecol(96,96,96);
       break;       
 }
 
 clear_to_color(screen,backcolor);
 makeborder(forecolor);
 if (screen_width==1280)
 {
  xzoom=yzoom=4;
  factor=4;
 }
 else
 {
  xzoom=yzoom=2;
  factor=3;
 }
 CharX=0; CharY=(1<<factor)<<1;
 
 switch (screentype)
 {
  case 0:
       makeborder(backcolor);
       xzoom>>=1;
       shifted=1;
       prints((uchar *)"*** hi65 - basic v10 ***"); Enter(); Enter();
       prints((uchar *)" (c) "); shifted=0; prints((uchar *)"2013-"); printn(year); shifted=1; CharX+=(1<<(factor)<<1);
       for (i=84; i<88; i++)
        Cout(c65basicerrors[i]);
       Cout(c65basicerrors[23]); CharX+=(1<<(factor+1));
       Cout(c65basicerrors[4]); Cout(c65basicerrors[5]);
       Cout(c65basicerrors[462]); Cout(c65basicerrors[463]);
       Cout(c65basicerrors[187]); Cout(c65basicerrors[188]);
       Enter();
       break;
  case 16:
       forecolor=makecol(0,0,0);
       shifted=0;
       prints((uchar *)" hi65: high level commodore 65 emulator"); Enter();
       prints((uchar *)" (c) "); prints((uchar *)"2013-"); printn(year); CharX+=(1<<(factor)<<1);
       for (i=84; i<88; i++)
        Cout(c65basicerrors[i]);
       Cout(c65basicerrors[23]); CharX+=(1<<(factor+1));
       Cout(c65basicerrors[4]); Cout(c65basicerrors[5]);
       Cout(c65basicerrors[462]); Cout(c65basicerrors[463]);
       Cout(c65basicerrors[187]); Cout(c65basicerrors[188]);
       Enter(); Enter();
       break;
  case 20:
       xzoom<<=1;
       forecolor=makecol(59,33,217);
       shifted=0;
       prints((uchar *)"****  hi65  v6  ****"); Enter(); Enter();
       prints((uchar *)"basic v10"); Enter(); Enter();
       break;
  case 64:
       shifted=0;
       prints((uchar *)"       **** hi65 - basic v10 ****"); Enter(); Enter();
       prints((uchar *)" high level commodore 65 emulator   v6"); Enter(); Enter();
       break;
  case 128:
       shifted=0;
       prints((uchar *)" hi65: high level commodore 65 emulator"); Enter();
       prints((uchar *)"       basic 10.0     version 6"); Enter();
       prints((uchar *)"      (c) "); prints((uchar *)"2013-"); printn(year); CharX+=(1<<(factor)<<1);
       for (i=84; i<88; i++)
        Cout(c65basicerrors[i]);
       Cout(c65basicerrors[23]); CharX+=(1<<(factor+1));
       Cout(c65basicerrors[4]); Cout(c65basicerrors[5]);
       Cout(c65basicerrors[462]); Cout(c65basicerrors[463]);
       Cout(c65basicerrors[187]); Cout(c65basicerrors[188]);
       Enter();
       prints((uchar *)"          all rights reserved"); Enter(); Enter();
 }
 Ready();
}

void flashcursor(void)
{
 if (!tickscapped)
  rectfill(screen,CharX+screen_border_x,CharY+screen_border_y,CharX+(xzoom<<3)+screen_border_x,CharY+(yzoom<<3)+screen_border_y-1,forecolor);
 else if (tickscapped==30)
  rectfill(screen,CharX+screen_border_x,CharY+screen_border_y,CharX+(xzoom<<3)+screen_border_x,CharY+(yzoom<<3)+screen_border_y-1,backcolor);
}
